@@ -23,7 +23,8 @@
 #'   model.
 #' @param cross_validation_parameters Optional, parameter grid for learner
 #'   parameters, e.g. \code{list(n_estimators = c(100, 200, 500),
-#'   learning_rate = c(0.01, 0.1), max_depth = c(2, 3))}.
+#'   learning_rate = c(0.01, 0.1), max_depth = c(2, 3))}
+#'   or \code{"hyperband"} for supported models.
 #' @param model_name Optional, the prefix of the Platform modeling jobs.
 #'   It will have \code{" Train"} or \code{" Predict"} added to become the Script title.
 #' @param calibration Optional, if not \code{NULL}, calibrate output
@@ -48,18 +49,22 @@
 #'   the model was built.
 #' @param if_output_exists Action to take if the prediction table already exists. One of \code{"fail"}, \code{"append"}, \code{"drop"}, or \code{"truncate"}.
 #'   The default is \code{"fail"}.
-#' @param n_jobs  Number of concurrent Platform jobs to use for
-#'   multi-file / large table prediction.
+#' @param n_jobs  Number of concurrent Platform jobs to use for training and
+#'   validation, or multi-file / large table prediction.
 #' @param cpu_requested Optional, the number of CPU shares requested in the
-#'   Civis Platform for training jobs. 1024 shares = 1 CPU.
+#'   Civis Platform for training jobs or prediction child jobs.
+#'   1024 shares = 1 CPU.
 #' @param memory_requested Optional, the memory requested from Civis Platform
-#'   for training jobs, in MiB.
+#'   for training jobs or prediction child jobs, in MiB.
 #' @param disk_requested Optional, the disk space requested on Civis Platform
-#'   for training jobs, in GB.
+#'   for training jobs or prediction child jobs, in GB.
 #' @param notifications Optional, model status notifications. See
 #'   \code{\link{scripts_post_custom}} for further documentation about email
 #'   and URL notification.
 #' @param polling_interval Check for job completion every this number of seconds.
+#' @param validation_data Optional, source for validation data. There are
+#'   currently two options: \code{train} (the default), which uses training
+#'   data for validation, and \code{skip}, which skips the validation step.
 #' @param verbose Optional, If \code{TRUE}, supply debug outputs in Platform
 #'   logs and make prediction child jobs visible.
 #' @param \dots Unused
@@ -71,7 +76,11 @@
 #' column. The \code{"sparse_*"} models include a LASSO regression step
 #' (using \code{glmnet}) to do feature selection before passing data to the
 #' final model. In some models, CivisML uses default parameters from those in
-#' \href{http://scikit-learn.org/stable/}{Scikit-Learn}.
+#' \href{http://scikit-learn.org/stable/}{Scikit-Learn}, as indicated in the "Altered Defaults" column.
+#' All models also have \code{random_state=42}. Note that \code{"multilayer_perceptron_classifier"}
+#' and \code{"multilayer_perceptron_regressor"} can only be used with
+#' hyperband.
+#'
 #' Specific workflows can also be called directly using the R workflow functions.
 #'
 #' \tabular{rrrrr}{
@@ -80,15 +89,51 @@
 #'  \code{gradient_boosting_classifier} \tab	\code{\link{civis_ml_gradient_boosting_classifier}} \tab classification \tab	\href{http://scikit-learn.org/stable/modules/generated/sklearn.ensemble.GradientBoostingClassifier.html}{GradientBoostingClassifier} \tab	\code{n_estimators=500, max_depth=2} \cr
 #'  \code{random_forest_classifier} \tab	\code{\link{civis_ml_random_forest_classifier}} \tab classification \tab	\href{http://scikit-learn.org/stable/modules/generated/sklearn.ensemble.RandomForestClassifier.html}{RandomForestClassifier} \tab	\code{n_estimators=500} \cr
 #'  \code{extra_trees_classifier} \tab	\code{\link{civis_ml_extra_trees_classifier}} \tab classification \tab	\href{http://scikit-learn.org/stable/modules/generated/sklearn.ensemble.ExtraTreesClassifier.html}{ExtraTreesClassifier} \tab	\code{n_estimators=500} \cr
+#'  \code{multilayer_perceptron_classifier} \tab \tab classification \tab \href{https://github.com/civisanalytics/muffnn}{MLPClassifier} \tab \cr
+#'  \code{stacking_classifier} \tab \tab classification  \tab \href{https://github.com/civisanalytics/civisml-extensions}{StackedClassifier}\tab \cr
 #'  \code{sparse_linear_regressor} \tab \code{\link{civis_ml_sparse_linear_regressor}} \tab	regression \tab	\href{http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.LinearRegression.html}{LinearRegression} \tab \cr
 #'  \code{sparse_ridge_regressor} \tab	\code{\link{civis_ml_sparse_ridge_regressor}} \tab regression \tab	\href{http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.Ridge.html}{Ridge} \tab \cr
 #'  \code{gradient_boosting_regressor}	\tab \code{\link{civis_ml_gradient_boosting_regressor}} \tab regression \tab \href{http://scikit-learn.org/stable/modules/generated/sklearn.ensemble.GradientBoostingRegressor.html}{GradientBoostingRegressor} \tab \code{n_estimators=500, max_depth=2} \cr
 #'  \code{random_forest_regressor}	\tab \code{\link{civis_ml_random_forest_regressor}} \tab regression \tab \href{http://scikit-learn.org/stable/modules/generated/sklearn.ensemble.RandomForestRegressor.html}{RandomForestRegressor} \tab \code{n_estimators=500} \cr
 #'  \code{extra_trees_regressor} \tab \code{\link{civis_ml_extra_trees_regressor}} \tab regression	\tab \href{http://scikit-learn.org/stable/modules/generated/sklearn.ensemble.ExtraTreesRegressor.html}{ExtraTreesRegressor} \tab \code{n_estimators=500} \cr
+#'  \code{multilayer_perceptron_regressor} \tab \tab regression \tab \href{https://github.com/civisanalytics/muffnn}{MLPRegressor} \tab \cr
+#'  \code{stacking_regressor} \tab \tab regression  \tab \href{https://github.com/civisanalytics/civisml-extensions}{StackedRegressor}\tab \cr
 #' }
 #' Model names can be easily accessed using the global variables \code{CIVIS_ML_REGRESSORS} and \code{CIVIS_ML_CLASSIFIERS}.
+#' @section Stacking:
 #'
+#' The \code{"stacking_classifier"} model stacks
+#' together the \code{"sparse_logistic"}, \code{"gradient_boosting_classifier"},
+#' and \code{"random_forest_classifier"} models, using altered defaults as
+#' listed for each in the \code{"Altered Defaults"} column of the table
+#' above. The models are combined using a pipeline
+#' containing a \href{http://scikit-learn.org/stable/modules/generated/sklearn.preprocessing.Normalizer.html#sklearn.preprocessing.Normalizer}{Normalizer}
+#' step, followed by a \href{http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.LogisticRegressionCV.html}{LogisticRegressionCV}
+#' with \code{penalty='l2'} and \code{tol=1e-08}. The
+#' \code{"stacking_regressor"} works similarly, stacking together the
+#' \code{"sparse_linear_regressor"}, \code{"gradient_boosting_regressor"},
+#' and \code{"random_forest_regressor"} models, and combining them using
+#' \href{https://github.com/civisanalytics/civisml-extensions}{NonNegativeLinearRegression}.
 #'
+#' @section Hyperparameter Tuning:
+#' You can tune hyperparameters using one of two methods: grid search or
+#' hyperband. CivisML will perform grid search if you pass a list
+#' of hyperparameters to the \code{cross_validation_parameters} parameter, where list elements are
+#' hyperparameter names, and the values are vectors of hyperparameter
+#' values to grid search over. You can run hyperparameter optimization in parallel by
+#' setting the \code{n_jobs}
+#' parameter to however many jobs you would like to run in
+#' parallel. \code{n_jobs} defaults to 1 (no parallelization).
+#'
+#'  \href{https://arxiv.org/abs/1603.06560}{Hyperband}
+#' is an efficient approach to hyperparameter optimization, and
+#' recommended over grid search where possible. CivisML will perform
+#' hyperband optimization if you pass the string \code{"hyperband"} to
+#' \code{cross_validation_parameters}. Hyperband is currently only supported for the following models:
+#' \code{"gradient_boosting_classifier"}, \code{"random_forest_classifier"},
+#' \code{"extra_trees_classifier"}, \code{"multilayer_perceptron_classifier"},
+#' \code{"gradient_boosting_regressor"}, \code{"random_forest_regressor"},
+#' \code{"extra_trees_regressor"}, and \code{"multilayer_perceptron_regressor"}.
 #' @section Data Sources:
 #'
 #' For building models with \code{civis_ml}, the training data can reside in
@@ -146,9 +191,9 @@
 #'  A list containing the following elements:
 #'   \itemize{
 #'   \item run list, metadata about the run.
-#'   \item data list, metdata about the training data.
+#'   \item data list, metadata about the training data.
 #'   \item model list, the fitted scikit-learn model.
-#'   \item metrics empy list.
+#'   \item metrics empty list.
 #'   \item warnings list.
 #'   \item data_platform list, training data location.
 #'   }}
@@ -216,6 +261,8 @@ civis_ml <- function(x,
                      disk_requested = NULL,
                      notifications = NULL,
                      polling_interval = NULL,
+                     validation_data = c('train', 'skip'),
+                     n_jobs = NULL,
                      verbose = FALSE) {
 
   UseMethod("civis_ml", x)
@@ -240,9 +287,12 @@ civis_ml.data.frame <- function(x,
                                 disk_requested = NULL,
                                 notifications = NULL,
                                 polling_interval = NULL,
+                                validation_data = c('train', 'skip'),
+                                n_jobs = NULL,
                                 verbose = FALSE) {
 
   oos_scores_if_exists <- match.arg(oos_scores_if_exists)
+  validation_data <- match.arg(validation_data)
 
   oos_scores_db_id <- NULL
   if (!is.null(oos_scores_db)) {
@@ -268,6 +318,8 @@ civis_ml.data.frame <- function(x,
                        cpu_requested = cpu_requested,
                        memory_requested = memory_requested,
                        disk_requested = disk_requested,
+                       validation_data = validation_data,
+                       n_jobs = n_jobs,
                        notifications = notifications,
                        verbose = verbose)
 }
@@ -291,9 +343,12 @@ civis_ml.civis_table <- function(x,
                                  disk_requested = NULL,
                                  notifications = NULL,
                                  polling_interval = NULL,
+                                 validation_data = c('train', 'skip'),
+                                 n_jobs = NULL,
                                  verbose = FALSE) {
 
   oos_scores_if_exists <- match.arg(oos_scores_if_exists)
+  validation_data <- match.arg(validation_data)
 
   oos_scores_db_id <- NULL
   if (!is.null(oos_scores_db)) {
@@ -319,6 +374,8 @@ civis_ml.civis_table <- function(x,
                        cpu_requested = cpu_requested,
                        memory_requested = memory_requested,
                        disk_requested = disk_requested,
+                       validation_data = validation_data,
+                       n_jobs = n_jobs,
                        notifications = notifications,
                        verbose = verbose)
 }
@@ -342,9 +399,12 @@ civis_ml.civis_file <- function(x,
                                 disk_requested = NULL,
                                 notifications = NULL,
                                 polling_interval = NULL,
+                                validation_data = c('train', 'skip'),
+                                n_jobs = NULL,
                                 verbose = FALSE) {
 
   oos_scores_if_exists <- match.arg(oos_scores_if_exists)
+  validation_data <- match.arg(validation_data)
 
   oos_scores_db_id <- NULL
   if (!is.null(oos_scores_db)) {
@@ -367,6 +427,8 @@ civis_ml.civis_file <- function(x,
                        cpu_requested = cpu_requested,
                        memory_requested = memory_requested,
                        disk_requested = disk_requested,
+                       validation_data = validation_data,
+                       n_jobs = n_jobs,
                        notifications = notifications,
                        verbose = verbose)
 }
@@ -390,9 +452,12 @@ civis_ml.character <- function(x,
                                disk_requested = NULL,
                                notifications = NULL,
                                polling_interval = NULL,
+                               validation_data = c('train', 'skip'),
+                               n_jobs = NULL,
                                verbose = FALSE) {
 
   oos_scores_if_exists <- match.arg(oos_scores_if_exists)
+  validation_data <- match.arg(validation_data)
 
   oos_scores_db_id <- NULL
   if (!is.null(oos_scores_db)) {
@@ -416,6 +481,8 @@ civis_ml.character <- function(x,
                        cpu_requested = cpu_requested,
                        memory_requested = memory_requested,
                        disk_requested = disk_requested,
+                       validation_data = validation_data,
+                       n_jobs = n_jobs,
                        notifications = notifications,
                        verbose = verbose)
 }
@@ -440,6 +507,9 @@ create_and_run_model <- function(file_id = NULL,
                                  cpu_requested = NULL,
                                  memory_requested = NULL,
                                  disk_requested = NULL,
+                                 polling_interval = NULL,
+                                 validation_data = NULL,
+                                 n_jobs = NULL,
                                  notifications = NULL,
                                  verbose = FALSE) {
 
@@ -448,7 +518,6 @@ create_and_run_model <- function(file_id = NULL,
     TARGET_COLUMN = paste(dependent_variable, collapse = " "),
     PRIMARY_KEY = primary_key,
     PARAMS = jsonlite::toJSON(parameters, auto_unbox = TRUE, null = "null"),
-    CVPARAMS = jsonlite::toJSON(cross_validation_parameters, null = "null"),
     IF_EXISTS = oos_scores_if_exists,
     TABLE_NAME = table_name,
     # We unclass the file_id here b/c jsonlite::toJSON does not know how to
@@ -470,6 +539,25 @@ create_and_run_model <- function(file_id = NULL,
     }
   }
 
+  if (!is.null(cross_validation_parameters)) {
+    hyperband <- identical(cross_validation_parameters, "hyperband")
+    hyperband_not_supported <- model_type %in% c("sparse_logistic", "sparse_linear_regressor",
+                                                 "stacking_regressor")
+    if (hyperband & hyperband_not_supported) {
+      stop(paste0("cross_validation_parameters = \"hyperband\" not supported for ", model_type))
+    }
+    is_mlp <- model_type %in% c("multilayer_perceptron_regressor", "multilayer_perceptron_classifier")
+    if (is_mlp & !hyperband) {
+      stop("cross_validation_parameters = \"hyperband\" is required for ", model_type)
+    }
+    if (hyperband) {
+      # need to escape quotes to get the string to work
+      args[["CVPARAMS"]] <- '\"hyperband\"'
+    } else {
+      args[["CVPARAMS"]] <- jsonlite::toJSON(cross_validation_parameters, null = "null")
+    }
+  }
+
   if (!is.null(calibration)) {
     if (!(calibration %in% c("sigmoid", "isotonic"))) {
       stop("calibration must be 'sigmoid', 'isotonic', or NULL.")
@@ -515,6 +603,14 @@ create_and_run_model <- function(file_id = NULL,
     args[["REQUIRED_DISK_SPACE"]] <- disk_requested
   }
 
+  if (!is.null(validation_data)) {
+    args[["VALIDATION_DATA"]] <- validation_data
+  }
+
+  if (!is.null(n_jobs)) {
+    args[["N_JOBS"]] <- n_jobs
+  }
+
   args <- I(args)  # We don't want any conversions and by toJSON.
 
   job_name <- NULL
@@ -524,11 +620,14 @@ create_and_run_model <- function(file_id = NULL,
 
   tmpl_id <- getOption("civis.ml_train_template_id")
   run <- run_model(template_id = tmpl_id, name = job_name, arguments = args,
-                   notifications = notifications, verbose = verbose)
+                   notifications = notifications,
+                   polling_interval = polling_interval,
+                   verbose = verbose)
   civis_ml_fetch_existing(run$job_id, run$run_id)
 }
 
-run_model <- function(template_id, name, arguments, notifications, verbose) {
+run_model <- function(template_id, name, arguments, notifications,
+                      polling_interval, verbose) {
   script_args <- list(
     from_template_id = template_id,
     arguments = arguments
@@ -547,7 +646,7 @@ run_model <- function(template_id, name, arguments, notifications, verbose) {
   job <- do.call(scripts_post_custom, script_args)
   run <- scripts_post_custom_runs(job$id)
   tryCatch(await(scripts_get_custom_runs, id = job$id, run_id = run$id,
-                    .verbose = verbose),
+                 .interval = polling_interval, .verbose = verbose),
           civis_error = function(e) stop(civis_ml_error(e)),
           error = function(e) stop(e))
   list(job_id = job$id, run_id = run$id)
@@ -630,6 +729,9 @@ predict.civis_ml <- function(object,
                              output_db = NULL,
                              if_output_exists = c('fail', 'append', 'drop', 'truncate'),
                              n_jobs = NULL,
+                             cpu_requested= NULL,
+                             memory_requested= NULL,
+                             disk_requested = NULL,
                              polling_interval = NULL,
                              verbose = FALSE,
                              ...) {
@@ -649,12 +751,16 @@ predict.civis_ml <- function(object,
   pred_args <- list(
     train_job_id = object$job$id,
     train_run_id = object$run$id,
+    template_id = get_predict_template_id(object),
     primary_key = primary_key,
     output_table = output_table,
     output_db_id = output_db_id,
     if_output_exists = if_output_exists,
     model_name = model_name,
     n_jobs = n_jobs,
+    cpu_requested= cpu_requested,
+    memory_requested= memory_requested,
+    disk_requested = disk_requested,
     polling_interval = polling_interval,
     verbose = verbose
   )
@@ -670,7 +776,6 @@ predict.civis_ml <- function(object,
     file_id <- write_civis_file(newdata, "modelpipeline_data.csv")
     pred_args[["file_id"]] <- file_id
   }
-
   if (inherits(newdata, "civis_file")) {
     # See above, we need to strip class attribute for jsonlite::toJSON.
     pred_args[["file_id"]] <- unclass(newdata)
@@ -693,6 +798,7 @@ predict.civis_ml <- function(object,
 
 create_and_run_pred <- function(train_job_id = NULL,
                                 train_run_id = NULL,
+                                template_id = NULL,
                                 file_id = NULL,
                                 table_name = NULL,
                                 database_id = NULL,
@@ -705,6 +811,9 @@ create_and_run_pred <- function(train_job_id = NULL,
                                 if_output_exists = NULL,
                                 model_name = NULL,
                                 n_jobs = NULL,
+                                cpu_requested= NULL,
+                                memory_requested= NULL,
+                                disk_requested = NULL,
                                 polling_interval = NULL,
                                 notifications = NULL,
                                 verbose = FALSE) {
@@ -740,10 +849,20 @@ create_and_run_pred <- function(train_job_id = NULL,
     args[["MANIFEST"]] <- manifest
   }
 
-  if (!is.null(n_jobs) && n_jobs == 1) {
-    args[["REQUIRED_CPU"]] <- 1024
-    args[["REQUIRED_MEMORY"]] <- 3000
-    args[["REQUIRED_DISK_SPACE"]] <- 30
+  if (!is.null(n_jobs)) {
+    args[["N_JOBS"]] <- n_jobs
+  }
+
+  if (!is.null(cpu_requested)) {
+    args[["CPU"]] <- cpu_requested
+  }
+
+  if (!is.null(memory_requested)) {
+    args[["MEMORY"]] <- memory_requested
+  }
+
+  if (!is.null(disk_requested)) {
+    args[["DISK_SPACE"]] <- disk_requested
   }
 
   args <- I(args)
@@ -753,9 +872,10 @@ create_and_run_pred <- function(train_job_id = NULL,
     job_name <- paste0(model_name, " Predict")
   }
 
-  tmpl_id <- getOption("civis.ml_predict_template_id")
-  run <- run_model(template_id = tmpl_id, name = job_name, arguments = args,
-                   notifications = notifications, verbose = verbose)
+  run <- run_model(template_id = template_id, name = job_name, arguments = args,
+                   notifications = notifications,
+                   polling_interval = polling_interval,
+                   verbose = verbose)
   fetch_predict_results(run$job_id, run$run_id)
 }
 
