@@ -2,15 +2,37 @@
 
 #' List of civis_ml regression models.
 #' @export
-CIVIS_ML_REGRESSORS <- c("sparse_linear_regressor", "sparse_ridge_regressor",
-                        "gradient_boosting_regressor","random_forest_regressor",
-                        "extra_trees_regressor")
+CIVIS_ML_REGRESSORS <- c("sparse_linear_regressor",
+                         "sparse_ridge_regressor",
+                         "gradient_boosting_regressor",
+                         "random_forest_regressor",
+                         "extra_trees_regressor",
+                         "multilayer_perceptron_regressor",
+                         "stacking_regressor")
 
 #' List of classification models.
 #' @export
-CIVIS_ML_CLASSIFIERS <- c("sparse_logistic", "gradient_boosting_classifier",
-                         "random_forest_classifier",
-                         "extra_trees_classifier")
+CIVIS_ML_CLASSIFIERS <- c("sparse_logistic",
+                          "gradient_boosting_classifier",
+                          "random_forest_classifier",
+                          "extra_trees_classifier",
+                          "multilayer_perceptron_classifier",
+                          "stacking_classifier")
+
+CIVIS_ML_TEMPLATE_IDS <- data.frame(
+  id = c(9112, 9113, 9968, 9969),
+  version = c(1.1, 1.1, 2.0, 2.0),
+  name = c("train", "predict", "train", "predict"),
+  stringsAsFactors = FALSE
+)
+
+# returns a version compatible template id for a given training model without API calls.
+get_predict_template_id <- function(m) {
+  train_id <- m$job$fromTemplateId
+  this_version <- CIVIS_ML_TEMPLATE_IDS[CIVIS_ML_TEMPLATE_IDS$id == train_id, "version"]
+  CIVIS_ML_TEMPLATE_IDS[CIVIS_ML_TEMPLATE_IDS$version == this_version &
+                        CIVIS_ML_TEMPLATE_IDS$name == "predict", "id"]
+}
 
 #' @export
 print.civis_ml_classifier <- function(x, digits = 4, ...) {
@@ -28,20 +50,23 @@ print.civis_ml_classifier <- function(x, digits = 4, ...) {
   cat(url, fill = TRUE)
   cat("Job id: ", job_id, " Run id: ", run_id, "\n", fill = TRUE)
 
-  if (any(is_multiclass(x)) & is_multitarget(x)) {
-    tabs <- mapply(class_metric_table, p_cor, class_names, aucs)
-  } else if (is_multiclass(x)) {
-      tabs <- list(class_metric_table(p_cor, class_names, aucs))
-  } else {
-    cat("AUC: ", signif(aucs, digits = digits), fill = TRUE)
-    tabs <- list(class_metric_table(p_cor, class_names))
+  if (!is.null(x$metrics)) {
+    if (any(is_multiclass(x)) & is_multitarget(x)) {
+      tabs <- mapply(class_metric_table, p_cor, class_names, aucs)
+    } else if (is_multiclass(x)) {
+        tabs <- list(class_metric_table(p_cor, class_names, aucs))
+    } else {
+      cat("AUC: ", signif(aucs, digits = digits), fill = TRUE)
+      tabs <- list(class_metric_table(p_cor, class_names))
+    }
+
+    for (i in seq_along(tabs)) {
+      cat(paste0(dv_names[i], ":"), fill = TRUE)
+      print(signif(tabs[[i]], digits = digits))
+      cat("\n")
+    }
   }
 
-  for (i in seq_along(tabs)) {
-    cat(paste0(dv_names[i], ":"), fill = TRUE)
-    print(signif(tabs[[i]], digits = digits))
-    cat("\n")
-  }
   invisible(x)
 }
 
@@ -69,10 +94,12 @@ print.civis_ml_regressor <- function(x, digits = 4, ...) {
   cat("Job id: ", job_id, " Run id: ", run_id, "\n", fill = TRUE)
 
   dv_names <- x$model_info$data$target_columns
-  tab <- with(x$metrics$metrics, t(cbind(r_squared, rmse, mad)))
-  colnames(tab) <- dv_names
-  rownames(tab) <- c( "R-squared", "RMSE", "MAD")
-  print(signif(tab, digits = digits))
+  if (!is.null(x$metrics)) {
+    tab <- with(x$metrics$metrics, t(cbind(r_squared, rmse, mad)))
+    colnames(tab) <- dv_names
+    rownames(tab) <- c( "R-squared", "RMSE", "MAD")
+    print(signif(tab, digits = digits))
+  }
   invisible(x)
 }
 
@@ -138,9 +165,9 @@ get_metric <- function(model, name = NULL) {
 
 get_model_data <- function(model, name = NULL) {
   if (!is.null(name)) {
-    model$metrics$data[[name]]
+    model$model_info$data[[name]]
   } else {
-    model$metrics$data
+    model$model_info$data
   }
 }
 
