@@ -19,7 +19,6 @@ test_that("calls scripts_post_custom", {
   fake_scripts_post_custom_runs <- mock(list(id = 888))
   fake_scripts_get_custom_runs <- mock(list(state = "running"), list(state = "succeeded"))
   fake_civis_ml_fetch_existing <- mock(NULL)
-  fake_getOption <- mock(1111, cycle = TRUE)
 
   with_mock(
     `civis::get_database_id` = fake_get_database_id,
@@ -27,7 +26,6 @@ test_that("calls scripts_post_custom", {
     `civis::scripts_post_custom_runs` = fake_scripts_post_custom_runs,
     `civis::scripts_get_custom_runs` = fake_scripts_get_custom_runs,
     `civis::civis_ml_fetch_existing` = fake_civis_ml_fetch_existing,
-    `base::getOption` = fake_getOption,
 
     tbl <- civis_table(table_name = "schema.table",
                        database_name = "a_database",
@@ -53,11 +51,13 @@ test_that("calls scripts_post_custom", {
              notifications = list(successEmailSubject = "A success",
                                   successEmailAddresses = c("user@example.com")),
              polling_interval = 5,
+             validation_data = "skip",
+             n_jobs = 9,
              verbose = FALSE)
   )
 
   script_args <- mock_args(fake_scripts_post_custom)[[1]]
-  expect_equal(script_args$from_template_id, 1111)
+  expect_equal(script_args$from_template_id, getOption("civis.ml_train_template_id"))
   expect_equal(script_args$name, "awesome civisml Train")
   expect_equal(script_args$notifications, list(successEmailSubject = "A success",
                                                successEmailAddresses = c("user@example.com")))
@@ -84,6 +84,8 @@ test_that("calls scripts_post_custom", {
   expect_equal(ml_args$REQUIRED_CPU, 1111)
   expect_equal(ml_args$REQUIRED_MEMORY, 9096)
   expect_equal(ml_args$REQUIRED_DISK_SPACE, 9)
+  expect_equal(ml_args$VALIDATION_DATA, "skip")
+  expect_equal(ml_args$N_JOBS, 9)
 
   # Make sure we started the job.
   expect_args(fake_scripts_post_custom_runs, 1, 999)
@@ -94,15 +96,11 @@ test_that("calls scripts_post_custom", {
 })
 
 test_that("calls civis_ml.data.frame for local df", {
-  fake_write_csv <- mock(NULL)
-  fake_temp_file <- mock("fake_temp_path")
   fake_write_civis_file <- mock(1234)
   fake_get_database_id <- mock(456)
   fake_create_and_run_model <- mock(NULL)
 
   with_mock(
-    `utils::write.csv` = fake_write_csv,
-    `base::tempfile` = fake_temp_file,
     `civis::write_civis_file` = fake_write_civis_file,
     `civis::get_database_id` = fake_get_database_id,
     `civis::create_and_run_model` = fake_create_and_run_model,
@@ -112,10 +110,6 @@ test_that("calls civis_ml.data.frame for local df", {
              dependent_variable = "the_target_column",
              primary_key = "the_pk_column"),
 
-    expect_args(fake_write_civis_file, 1,
-                path = "fake_temp_path",
-                name = "modelpipeline_data.csv"),
-
     expect_args(fake_create_and_run_model, 1,
                 file_id = 1234,
                 dependent_variable = "the_target_column",
@@ -133,6 +127,8 @@ test_that("calls civis_ml.data.frame for local df", {
                 cpu_requested = NULL,
                 memory_requested = NULL,
                 disk_requested = NULL,
+                validation_data = 'train',
+                n_jobs = NULL,
                 notifications = NULL,
                 verbose = FALSE)
   )
@@ -179,6 +175,8 @@ test_that("calls civis_ml.civis_table for table_name", {
               cpu_requested = NULL,
               memory_requested = NULL,
               disk_requested = NULL,
+              validation_data = 'train',
+              n_jobs = NULL,
               notifications = NULL,
               verbose = FALSE)
 })
@@ -214,6 +212,8 @@ test_that("calls civis_ml.civis_file for file_id", {
               cpu_requested = NULL,
               memory_requested = NULL,
               disk_requested = NULL,
+              validation_data = 'train',
+              n_jobs = NULL,
               notifications = NULL,
               verbose = FALSE)
 })
@@ -255,6 +255,8 @@ test_that("calls civis_ml.character for local csv", {
               cpu_requested = NULL,
               memory_requested = NULL,
               disk_requested = NULL,
+              validation_data = 'train',
+              n_jobs = NULL,
               notifications = NULL,
               verbose = FALSE)
 })
@@ -301,11 +303,14 @@ test_that("raises error if multioutput not supported", {
 # Predict
 context("predict.civis_ml")
 
+current <- tail(CIVIS_ML_TEMPLATE_IDS, n = 2)
+
 fake_model <- structure(
   list(
     job = list(
       id = 123,
       name = "model_task",
+      fromTemplateId = current[current$name == "train", "id"],
       arguments = list(
         PRIMARY_KEY = "training_primary_key"
       )
@@ -321,7 +326,6 @@ test_that("calls scripts_post_custom", {
   fake_scripts_post_custom_runs <- mock(list(id = 888))
   fake_scripts_get_custom_runs <- mock(list(state = "running"), list(state = "succeeded"))
   fake_fetch_predict_results <- mock(NULL)
-  fake_getOption <- mock(1111, cycle = TRUE)
 
   with_mock(
     `civis::get_database_id` = fake_get_database_id,
@@ -329,7 +333,6 @@ test_that("calls scripts_post_custom", {
     `civis::scripts_post_custom_runs` = fake_scripts_post_custom_runs,
     `civis::scripts_get_custom_runs` = fake_scripts_get_custom_runs,
     `civis::fetch_predict_results` = fake_fetch_predict_results,
-    `base::getOption` = fake_getOption,
 
     tbl <- civis_table(table_name = "schema.table",
                        database_name = "a_database",
@@ -342,12 +345,15 @@ test_that("calls scripts_post_custom", {
             output_db = "score_database",
             if_output_exists = "append",
             n_jobs = 10,
+            cpu_requested = 2000,
+            memory_requested = 10,
+            disk_requested = 15,
             polling_interval = 5,
             verbose = TRUE)
   )
 
   script_args <- mock_args(fake_scripts_post_custom)[[1]]
-  expect_equal(script_args$from_template_id, 1111)
+  expect_equal(script_args$from_template_id, get_predict_template_id(fake_model))
   expect_equal(script_args$name, "model_task Predict")
 
   # These are template args/params:
@@ -358,6 +364,9 @@ test_that("calls scripts_post_custom", {
   expect_equal(pred_args$PRIMARY_KEY, "row_number")
   expect_equal(pred_args$IF_EXISTS, "append")
   expect_equal(pred_args$N_JOBS, 10)
+  expect_equal(pred_args$CPU, 2000)
+  expect_equal(pred_args$MEMORY, 10)
+  expect_equal(pred_args$DISK_SPACE, 15)
   expect_equal(pred_args$DEBUG, TRUE)
   expect_equal(pred_args$CIVIS_FILE_ID, NULL)
   expect_equal(pred_args$TABLE_NAME, "schema.table")
@@ -392,34 +401,29 @@ test_that("uses training primary_key by default", {
 })
 
 test_that("uploads local df and passes a file_id", {
-  fake_write_csv <- mock(NULL)
-  fake_temp_file <- mock("fake_temp_path")
   fake_write_civis_file <- mock(1234)
   fake_create_and_run_pred <- mock(NULL)
 
   with_mock(
-    `utils::write.csv` = fake_write_csv,
-    `base::tempfile` = fake_temp_file,
     `civis::write_civis_file` = fake_write_civis_file,
     `civis::create_and_run_pred` = fake_create_and_run_pred,
 
     predict(fake_model, iris, primary_key = NULL)
   )
 
-  expect_args(fake_write_csv, 1,
-              iris,
-              file = "fake_temp_path",
-              row.names = FALSE)
-
   expect_args(fake_create_and_run_pred, 1,
               train_job_id = fake_model$job$id,
               train_run_id = fake_model$run$id,
+              template_id = get_predict_template_id(fake_model),
               primary_key = NULL,
               output_table = NULL,
               output_db_id = NULL,
               if_output_exists = 'fail',
               model_name = "model_task",
               n_jobs = NULL,
+              cpu_requested = NULL,
+              memory_requested = NULL,
+              disk_requested = NULL,
               polling_interval = NULL,
               verbose = FALSE,
               file_id = 1234)
@@ -443,12 +447,16 @@ test_that("uploads a local file and passes a file_id", {
   expect_args(fake_create_and_run_pred, 1,
               train_job_id = fake_model$job$id,
               train_run_id = fake_model$run$id,
+              template_id = get_predict_template_id(fake_model),
               primary_key = NULL,
               output_table = NULL,
               output_db_id = NULL,
               if_output_exists = 'fail',
               model_name = "model_task",
               n_jobs = NULL,
+              cpu_requested = NULL,
+              memory_requested = NULL,
+              disk_requested = NULL,
               polling_interval = NULL,
               verbose = FALSE,
               file_id = 561)
@@ -466,12 +474,16 @@ test_that("passes a file_id directly", {
   expect_args(fake_create_and_run_pred, 1,
               train_job_id = fake_model$job$id,
               train_run_id = fake_model$run$id,
+              template_id = get_predict_template_id(fake_model),
               primary_key = "training_primary_key",
               output_table = NULL,
               output_db_id = NULL,
               if_output_exists = 'fail',
               model_name = "model_task",
               n_jobs = NULL,
+              cpu_requested = NULL,
+              memory_requested = NULL,
+              disk_requested = NULL,
               polling_interval = NULL,
               verbose = FALSE,
               file_id = 1234)
@@ -489,12 +501,16 @@ test_that("passes a manifest file_id", {
   expect_args(fake_create_and_run_pred, 1,
               train_job_id = fake_model$job$id,
               train_run_id = fake_model$run$id,
+              template_id = get_predict_template_id(fake_model),
               primary_key = NULL,
               output_table = NULL,
               output_db_id = NULL,
               if_output_exists = 'fail',
               model_name = "model_task",
               n_jobs = NULL,
+              cpu_requested = NULL,
+              memory_requested = NULL,
+              disk_requested = NULL,
               polling_interval = NULL,
               verbose = FALSE,
               manifest = 123)
@@ -521,12 +537,16 @@ test_that("passes table info", {
   expect_args(fake_create_and_run_pred, 1,
               train_job_id = fake_model$job$id,
               train_run_id = fake_model$run$id,
+              template_id = get_predict_template_id(fake_model),
               primary_key = NULL,
               output_table = NULL,
               output_db_id = NULL,
               if_output_exists = 'fail',
               model_name = "model_task",
               n_jobs = NULL,
+              cpu_requested = NULL,
+              memory_requested = NULL,
+              disk_requested = NULL,
               polling_interval = NULL,
               verbose = FALSE,
               table_name = "a_schema.table",
@@ -540,12 +560,10 @@ test_that("passes table info", {
 context("create_and_run_model")
 
 test_that("uses the correct template_id", {
-  fake_getOption <- mock(999999, cycle = TRUE)
   fake_run_model <- mock(list(job_id = 133, run_id = 244))
   fake_civis_ml_fetch_existing <- mock(NULL)
 
   with_mock(
-    `base::getOption` = fake_getOption,
     `civis::run_model` = fake_run_model,
     `civis::civis_ml_fetch_existing` = fake_civis_ml_fetch_existing,
 
@@ -553,19 +571,16 @@ test_that("uses the correct template_id", {
   )
 
   run_args <- mock_args(fake_run_model)[[1]]
-  expect_equal(run_args$template_id, 999999)
+  expect_equal(run_args$template_id, getOption("civis.ml_train_template_id"))
 })
 
 test_that("converts parameters arg to JSON string", {
-  fake_getOption <- mock(999999, cycle = TRUE)
   fake_run_model <- mock(list(job_id = 133, run_id = 244))
   fake_civis_ml_fetch_existing <- mock(NULL)
 
   with_mock(
-    `base::getOption` = fake_getOption,
     `civis::run_model` = fake_run_model,
     `civis::civis_ml_fetch_existing` = fake_civis_ml_fetch_existing,
-
     create_and_run_model(file_id = 123, parameters = list(n_trees = 500, c = -1))
   )
 
@@ -574,16 +589,15 @@ test_that("converts parameters arg to JSON string", {
 })
 
 test_that("converts cross_validation_parameters to JSON string", {
-  fake_getOption <- mock(999999, cycle = TRUE)
   fake_run_model <- mock(list(job_id = 133, run_id = 244))
   fake_civis_ml_fetch_existing <- mock(NULL)
 
   with_mock(
-    `base::getOption` = fake_getOption,
     `civis::run_model` = fake_run_model,
     `civis::civis_ml_fetch_existing` = fake_civis_ml_fetch_existing,
 
     create_and_run_model(file_id = 123,
+                         model_type = "sparse_logistic",
                          cross_validation_parameters = list(n_trees = c(500, 250), c = -1))
   )
 
@@ -593,12 +607,10 @@ test_that("converts cross_validation_parameters to JSON string", {
 })
 
 test_that("converts fit_params to JSON string", {
-  fake_getOption <- mock(999999, cycle = TRUE)
   fake_run_model <- mock(list(job_id = 133, run_id = 244))
   fake_civis_ml_fetch_existing <- mock(NULL)
 
   with_mock(
-    `base::getOption` = fake_getOption,
     `civis::run_model` = fake_run_model,
     `civis::civis_ml_fetch_existing` = fake_civis_ml_fetch_existing,
 
@@ -611,15 +623,12 @@ test_that("converts fit_params to JSON string", {
 })
 
 test_that("space separates excluded_columns", {
-  fake_getOption <- mock(999999, cycle = TRUE)
   fake_run_model <- mock(list(job_id = 133, run_id = 244))
   fake_civis_ml_fetch_existing <- mock(NULL)
 
   with_mock(
-    `base::getOption` = fake_getOption,
     `civis::run_model` = fake_run_model,
     `civis::civis_ml_fetch_existing` = fake_civis_ml_fetch_existing,
-
     create_and_run_model(file_id = 132, excluded_columns = c("c1", "c2", "c3"))
   )
 
@@ -628,12 +637,10 @@ test_that("space separates excluded_columns", {
 })
 
 test_that("space separates target_column", {
-  fake_getOption <- mock(999999, cycle = TRUE)
   fake_run_model <- mock(list(job_id = 133, run_id = 244))
   fake_civis_ml_fetch_existing <- mock(NULL)
 
   with_mock(
-    `base::getOption` = fake_getOption,
     `civis::run_model` = fake_run_model,
     `civis::civis_ml_fetch_existing` = fake_civis_ml_fetch_existing,
 
@@ -646,15 +653,12 @@ test_that("space separates target_column", {
 })
 
 test_that("file_id is always numeric", {
-  fake_getOption <- mock(999999, cycle = TRUE)
   fake_run_model <- mock(list(job_id = 133, run_id = 244))
   fake_civis_ml_fetch_existing <- mock(NULL)
 
   with_mock(
-    `base::getOption` = fake_getOption,
     `civis::run_model` = fake_run_model,
     `civis::civis_ml_fetch_existing` = fake_civis_ml_fetch_existing,
-
     create_and_run_model(file_id = civis_file(132))
   )
 
@@ -662,48 +666,44 @@ test_that("file_id is always numeric", {
   expect_equal(run_args$arguments$CIVIS_FILE_ID, 132)
 })
 
+
+test_that("exceptions with hyperband correct", {
+  fake_run_model <- mock(list(job_id = 133, run_id = 244))
+  fake_civis_ml_fetch_existing <- mock(NULL)
+
+  with_mock(
+    `civis::run_model` = fake_run_model,
+    `civis::civis_ml_fetch_existing` = fake_civis_ml_fetch_existing,
+    err1 <-  "cross_validation_parameters = \"hyperband\" not supported for sparse_logistic",
+    expect_error(create_and_run_model(file_id = civis_file(132),
+                                      model_type = "sparse_logistic",
+                                      cross_validation_parameters = "hyperband"), err1),
+    err2 <-  "cross_validation_parameters = \"hyperband\" is required for multilayer_perceptron_regressor",
+    expect_error(create_and_run_model(file_id = civis_file(132),
+                                      model_type = "multilayer_perceptron_regressor",
+                                      cross_validation_parameters = list(a = 5)), err2)
+  )
+})
+
+
 ################################################################################
 # run predictions
 context("create_and_run_pred")
 
 test_that("uses the correct template_id", {
-  fake_getOption <- mock(8888, cycle = TRUE)
   fake_run_model <- mock(list(job_id = 133, run_id = 244))
   fake_fetch_predict_results <- mock(NULL)
 
   with_mock(
-    `base::getOption` = fake_getOption,
     `civis::run_model` = fake_run_model,
     `civis::fetch_predict_results` = fake_fetch_predict_results,
 
-    create_and_run_pred(train_job_id = 111, train_run_id = 222)
+    create_and_run_pred(train_job_id = 111, train_run_id = 222, template_id = 555),
+    run_args <- mock_args(fake_run_model)[[1]],
+    expect_equal(run_args$template_id, 555)
   )
-
-  run_args <- mock_args(fake_run_model)[[1]]
-  expect_equal(run_args$template_id, 8888)
 })
 
-test_that("adds resources when n_jobs = 1", {
-  fake_getOption <- mock(8888, cycle = TRUE)
-  fake_run_model <- mock(list(job_id = 133, run_id = 244))
-  fake_fetch_predict_results <- mock(NULL)
-
-  with_mock(
-    `base::getOption` = fake_getOption,
-    `civis::run_model` = fake_run_model,
-    `civis::fetch_predict_results` = fake_fetch_predict_results,
-
-    create_and_run_pred(train_job_id = 111, train_run_id = 222, n_jobs = 1)
-  )
-
-  run_args <- mock_args(fake_run_model)[[1]]
-  expect_equal(run_args$arguments$REQUIRED_CPU, 1024)
-  expect_equal(run_args$arguments$REQUIRED_MEMORY, 3000)
-  expect_equal(run_args$arguments$REQUIRED_DISK_SPACE, 30)
-})
-
-###############################################################################
-
 ################################################################################
 # fetch existing model
 context("civis_ml_fetch_existing")
@@ -783,7 +783,8 @@ test_that("fetch_logs.civis_ml_error works", {
     `civis::scripts_post_custom_runs` = function(...) NULL,
     `civis::scripts_get_custom_runs` = function(...) list(state = "failed", id = 1, run_id = 2, error = "msg"),
     `civis::fetch_logs.civis_ml_error` = function(...) list("A log message"),
-    e <- tryCatch(civis:::run_model(1234, name = "sparse_logistic", list(), list(), verbose = TRUE),
+    e <- tryCatch(civis:::run_model(1234, name = "sparse_logistic", list(), list(),
+                                    verbose = TRUE, polling_interval = NULL),
              error = function(e) e),
     log <- fetch_logs(e)[[1]],
     expect_equal(log, "A log message"))
@@ -849,24 +850,18 @@ context("fetch_oos_scores")
 
 test_that("it checks input type", {
   fake_must_fetch_output_file <- mock(NULL)
-  fake_read_csv <- mock(NULL)
 
   with_mock(
-    `utils::read.csv` = fake_read_csv,
     `civis::must_fetch_output_file` = fake_must_fetch_output_file,
-
     expect_error(fetch_oos_scores("not a model"), "is_civis_ml(model) is not TRUE", fixed = TRUE)
   )
 })
 
 test_that("it looks for predictions.csv.gz", {
-  fake_must_fetch_output_file <- mock(NULL)
-  fake_read_csv <- mock(NULL)
+  fake_must_fetch_output_file <- mock(textConnection(c("a, b, c")))
 
   with_mock(
-    `utils::read.csv` = fake_read_csv,
     `civis::must_fetch_output_file` = fake_must_fetch_output_file,
-
     fetch_oos_scores(structure(list(), class = "civis_ml"))
   )
 
@@ -875,19 +870,15 @@ test_that("it looks for predictions.csv.gz", {
 })
 
 test_that("it calls read.csv with extra args", {
-  fake_must_fetch_output_file <- mock("a_file.csv")
-  fake_read_csv <- mock(NULL)
+  fake_must_fetch_output_file <- mock(textConnection(c("a,b,c")))
 
-  with_mock(
-    `utils::read.csv` = fake_read_csv,
+  df <- with_mock(
     `civis::must_fetch_output_file` = fake_must_fetch_output_file,
-
-    fetch_oos_scores(structure(list(), class = "civis_ml"), stringsAsFactors = FALSE)
+    fetch_oos_scores(structure(list(), class = "civis_ml"),
+                           stringsAsFactors = FALSE, header = FALSE)
   )
-
-  csv_args <- mock_args(fake_read_csv)[[1]]
-  expect_equal(csv_args[[1]], "a_file.csv")
-  expect_equal(csv_args$stringsAsFactors, FALSE)
+  ans <- data.frame(V1 = "a", V2 = "b", V3 = "c", stringsAsFactors = FALSE)
+  expect_equal(df, ans)
 })
 
 
@@ -900,18 +891,16 @@ test_that("it checks input type", {
 
 test_that("it calls read.csv with extra args, and dowload_civis with correct id", {
   fake_read_csv <- mock(NULL)
-  fake_download_civis <- mock("path.csv")
+  fake_download_civis <- mock(textConnection(c("a,b,c")))
 
-  with_mock(
-    `utils::read.csv` = fake_read_csv,
+  df <- with_mock(
     `civis::fetch_predict_results` = function(...) list(model_info = list(output_file_ids = 1)),
     `civis::download_civis` = fake_download_civis,
-    fetch_predictions(structure(list(), class = "civis_ml_prediction"), stringsAsFactors = FALSE)
+    fetch_predictions(structure(list(), class = "civis_ml_prediction"),
+                      header = FALSE, stringsAsFactors = FALSE)
   )
-
-  csv_args <- mock_args(fake_read_csv)[[1]]
-  expect_equal(csv_args[[1]], "path.csv")
-  expect_equal(csv_args$stringsAsFactors, FALSE)
+  ans <- data.frame(V1 = "a", V2 = "b", V3 = "c", stringsAsFactors = FALSE)
+  expect_equal(df, ans)
 
   dl_args <- mock_args(fake_download_civis)[[1]]
   expect_equal(dl_args[[1]], 1)
